# -*- coding: utf-8 -*-

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import swat_em.config

# TODO: Test the config save without overwriting the users config in home folder!
#  def test1():
    #  config = swat_em.config.get_config()
    #  swat_em.config.save_config(config)
    #  config2 = swat_em.config.get_config(init = False)
    #  assert config == config2

#  if __name__ == '__main__':
    #  test1()






