# -*- coding: utf-8 -*-
"""


"""
import os
__title__ = 'SWAT-EM - Specific Winding Analyse Tool for electrical machines'
__version__ = '0.1.1'
__author__ = 'Martin Baun'
__license__ = 'GPLv3'
__copyright__ = 'Copyright 2018-2019 Martin Baun'

#  from .gui import MainWindow
#  import swat.gui as gui
__dir__ = os.path.dirname(os.path.abspath(__file__))

#  from PyQt5.QtWidgets import  QApplication
#  import sys


#  def run():
    #  app = QApplication(sys.argv)
    #  ex = gui.MainWindow()
    #  sys.exit(app.exec_())

    
